&ANALYZE-SUSPEND _VERSION-NUMBER UIB_v9r12
&ANALYZE-RESUME
&ANALYZE-SUSPEND _UIB-CODE-BLOCK _CUSTOM _DEFINITIONS Procedure 
/*------------------------------------------------------------------------
    File        : load/xloader.p 
    Purpose     : Monitors an FTP server for incoming files
                  and processes them.

    Syntax      :

    Description : For the full FTP specification
                  look for RFC959 on http://www.faqs.org
                  This code should be considered as still in development, 
                  as such it has been unit tested only.
                  It was not written to be a generic solution.
                  It is heavily based upon the example smtp process
                  available from the Porgress Knowledge base.
                  Use this software at your own risk!
                  Jason Howells 12/07/2001

    Author(s)   : Jason Howells
    Created     : 04/06/2001
    Notes       : Some FTP terms are used throughout:
                  PI  = Protocol Interpreter
                  DTP = Data Transfer Process
                  
                  An FTP Session consists of the following components
                  
                  Client                Server
                  |                          |
                  PI -------Port 21---------PI
                  |                          |
                  |                          |
                  DTP-------Port xxx-------DTP
                  |                          |
                  
                  Theory
                  ------
                  Commands travel across port 21, data is transferred across an additional 
                  data transfer port.
                  Normally the FTP client process(this process) sets up
                  a server for the DTP and waits for the server process
                  to contact it.
                  There is however a slightly easier way that is also 
                  preferable when working through a firewall, Passive mode.
                  Passive mode revereses the process stated above.
                  When the client contacts the server process it issues
                  as PASV command to invoke passive mode.
                  The reply to the PASV command is a string that contains the
                  host and a port address for the DTP on the server.
                  In this way we only have to connect a client socket to the server.
                  This process is repeated for each file transfer.
                  The FTP specification states that control of the data connection
                  (DTP) resides wholly with the server process.
                  However during my testing I found it necessary to close the 
                  data connection from the client side before quiting the FTP session.
                  
                  Practice
                  --------
                  This program functions as follows:
                   1. Connects to the FTP server on port 21 and recieves a response.
                   2. Logs in using the gievn user name and password.
                   3. Sets the transfer type to I (to avoid CRLF's getting appended to
                      the end of every sodding line!)
                   4. Makes a list of all the xml files present.
                   5. Works through the list getting one file at a time and streaming it 
                      into a memory pointer.
                   6. This memory pointer is then passed to the Xload function that parses 
                      the XML document.
                   7. Once the document is parsed and loaded into the database, the remote file is
                      then moved into a different directory for archiving purposes.
                   8. The next file is then processed.
                   9. When the file list is empty, the program tries to retrive another list
                      if the response is '550 No such file or directory', then there are no xml files to 
                      retrieve so the process quits and control returns to the main block
                      in this procedure.
                  10. The program then pauses before beginning the cycle again.
                  
                  FTP Commands Used (RFC959 contains the full explanation)
                  Command Description
                  ------- ------------------------------------------------------------------------
                  USER    Specify username for login
                  PASS    Specify password for login
                  RNFR    Rename from... name of file to rename
                  RNTO    Rename to... new name of file
                  NOOP    Does nothing... returns OK, used here to trip the event processing
                  RETR    Retrieve... gets the named file
                  NLST    Name List... gets a simple list of files for the specifed directory.
                  QUIT    Go on guess...
                                          
                                                                                                                
  ----------------------------------------------------------------------*/
/*          This .W file was created with the Progress AppBuilder.      */
/*----------------------------------------------------------------------*/

/* ***************************  Definitions  ************************** */
define variable lh-PISocket               as handle              no-undo.
define variable lh-DTPSocket              as handle              no-undo.
define variable li-currentState           as integer             no-undo.
define variable lc-user                   as character           no-undo.
define variable lc-password               as character           no-undo.
define variable lc-FTPserverHost          as character           no-undo.
define variable lm-document               as memptr              no-undo.
define variable li-idx                    as integer             no-undo.
define variable li-bytesToRead            as integer             no-undo.
define variable li-bytesRead              as integer             no-undo.
define variable li-currentFile            as integer             no-undo.
define variable li-cycleWait              as integer             no-undo.
define variable li-filesToLoad            as integer             no-undo.
define variable lc-loadedDir              as character           no-undo.

define variable ll-debug                  as logical             no-undo.
define variable lc-debugFile              as character           no-undo.
define variable lc-time                   as character           no-undo.
define variable lc-basefile               as character           no-undo.
define variable lc-log                    as character           no-undo. 
define variable lh-xLoadLib               as handle              no-undo.
define variable lh-wPublib                as handle              no-undo.
define variable lc-rootLoadDir            as character           no-undo.
define variable lc-fileName               as character           no-undo.
define variable lc-fileList               as character           no-undo.
define variable lc-listing                as character           no-undo.
define variable lc-currentFile            as character           no-undo.
define variable li-listBufferSize         as integer             no-undo.

define stream debug-stream.

&scoped-define TSTAMP string(TODAY,"99/99/9999") string(TIME,"HH:MM:SS")
&scoped-define TSTAMP1 "<event date=""" string(TODAY,"99999999") """ time=""" string(TIME,"HH:MM:SS") """>"
&scoped-define DEBUG if ll-debug then put stream debug-stream unformatted {&TSTAMP1}
&scoped-define PAD "                      ":U
&scoped-define VERSION 1.0

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME


&ANALYZE-SUSPEND _UIB-PREPROCESSOR-BLOCK 

/* ********************  Preprocessor Definitions  ******************** */

&Scoped-define PROCEDURE-TYPE Procedure
&Scoped-define DB-AWARE no



/* _UIB-PREPROCESSOR-BLOCK-END */
&ANALYZE-RESUME


/* ************************  Function Prototypes ********************** */

&IF DEFINED(EXCLUDE-chomp) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD chomp Procedure 
FUNCTION chomp RETURNS CHARACTER
  ( pc-string as char, 
    pi-i      as int)  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-debugEvent) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD debugEvent Procedure 
FUNCTION debugEvent RETURNS LOGICAL
  ( input pc-ID    as char,
    input pc-title as char,
    input pc-text  as char)  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-destroy) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD destroy Procedure 
FUNCTION destroy RETURNS LOGICAL
  ( /* void */ )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-DTPconnect) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD DTPconnect Procedure 
FUNCTION DTPconnect RETURNS LOGICAL
  ( pc-string as character )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-getFileList) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD getFileList Procedure 
FUNCTION getFileList RETURNS LOGICAL
  ( pm-buffer as memptr )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-initialise) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD initialise Procedure 
FUNCTION initialise RETURNS LOGICAL
  ( /* void */ )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-newDirName) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD newDirName Procedure 
FUNCTION newDirName returns character
    (input pc-path as char) FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-PIconnect) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD PIconnect Procedure 
FUNCTION PIconnect returns logical
  ( /* void */ )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-setBuffer) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD setBuffer Procedure 
FUNCTION setBuffer RETURNS LOGICAL
  ( pc-string as char )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-startDebug) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD startDebug Procedure 
FUNCTION startDebug returns logical
  ( /* void */ )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-stopDebug) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD stopDebug Procedure 
FUNCTION stopDebug RETURNS LOGICAL
  ( /* void */ )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-xload) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD xload Procedure 
FUNCTION xload returns logical
  ( pm-document as memptr )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF


/* *********************** Procedure Settings ************************ */

&ANALYZE-SUSPEND _PROCEDURE-SETTINGS
/* Settings for THIS-PROCEDURE
   Type: Procedure
   Allow: 
   Frames: 0
   Add Fields to: Neither
   Other Settings: CODE-ONLY COMPILE
 */
&ANALYZE-RESUME _END-PROCEDURE-SETTINGS

/* *************************  Create Window  ************************** */

&ANALYZE-SUSPEND _CREATE-WINDOW
/* DESIGN Window definition (used by the UIB) 
  CREATE WINDOW Procedure ASSIGN
         HEIGHT             = 15.76
         WIDTH              = 45.6.
/* END WINDOW DEFINITION */
                                                                        */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _CUSTOM _INCLUDED-LIB Procedure 
/* ************************* Included-Libraries *********************** */

{sys/sockFns.i}
{sys/cfgfns.i}
{sys/xmlfns.i}

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME


 


&ANALYZE-SUSPEND _UIB-CODE-BLOCK _CUSTOM _MAIN-BLOCK Procedure 


/* ***************************  Main Block  *************************** */
     

    if initialise() then
    MAIN:
    repeat on stop undo, leave 
           on quit undo, leave:

        if li-currentState = 999 then
            do:
                debugEvent("000","MAIN Block","Waiting for " + string(li-cycleWait) + " seconds").
                pause li-cycleWait no-message.                
                PIconnect().           
            end.
        else
        if li-currentState < 0 then
            leave MAIN.
       
        wait-for read-response of lh-PISocket.
    end.

    destroy().

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME


/* **********************  Internal Procedures  *********************** */

&IF DEFINED(EXCLUDE-DTPreadResponse) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE DTPreadResponse Procedure 
PROCEDURE DTPreadResponse :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
   define variable li-bytesAvail             as integer                 no-undo.
   define variable lc-statusCode             as character               no-undo.   
   define variable lc-Str                    as character               no-undo.
   define variable ll-ok                     as logical initial true    no-undo.
  
   li-bytesAvail = self:get-bytes-available().

   debugEvent("000","DTPreadResponse","(DTP) li-bytesAvail=" + string(li-bytesAvail)).
   
   if li-bytesAvail > 0 then
   do:         
       debugEvent("010","DTPreadResponse","(DTP) li-bytesAvail=" + string(li-bytesAvail)).

       if self:read(lm-document, li-bytesRead + 1, li-bytesAvail) then      
       do:
           li-bytesRead = li-bytesRead + li-bytesAvail.

           debugEvent("020","DTPreadResponse","(DTP) li-bytesRead=" 
                                               + string(li-bytesRead)
                                               + " "
                                               + string(truncate((li-bytesRead / li-bytesToRead) * 100,0)) 
                                               + "% Complete").

       end.

   end. /* ..of if li-bytesAvail > 0 then... */ 
   
END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-newState) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE newState Procedure 
PROCEDURE newState :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
    define input parameter pi-newState         as integer              no-undo.
    define input parameter pc-controlRequest   as character            no-undo.

    if pc-controlRequest ne "":U then
    do:
        socketWrite(lh-PISocket,pc-controlRequest + "~n":U).

        debugEvent("010","newState","pc-controlRequest=" + pc-controlRequest).
    end.

    debugEvent("020","newState","(State Change) from " + string(li-currentState) + " to " + string(pi-newState)).

    li-currentState = pi-newState.
    
end procedure.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-PIreadResponse) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PIreadResponse Procedure 
PROCEDURE PIreadResponse :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
   define variable li-bytesAvail             as integer                 no-undo.
   define variable lc-statusCode             as character               no-undo.
   define variable li-FTPStatusCode          as integer                 no-undo.
   define variable lc-controlResponse        as character               no-undo.

   li-bytesAvail = self:get-bytes-available().

   if li-bytesAvail > 0 then
   do:
      lc-controlResponse = socketRead(self:handle, li-bytesAvail).
      
      debugEvent("010","PIreadResponse","lc-controlResponse=" + chomp(lc-controlResponse,2)).
      
      li-FTPstatusCode = int(entry(1,lc-controlResponse,chr(32))) no-error.

      if error-status:error then
      do:
          li-currentState = -10.
          return.
      end.
                                                          
      case li-currentState:
          when 10 then
               if li-FTPstatusCode = 220 then 
               run newState(20, "USER " + lc-user).
               else 
               li-currentState = -20.
          when 20 then
               if li-FTPstatusCode = 331 then 
               run newState(30, "PASS " + lc-password).
               else 
               li-currentState = -30.
          when 30 then
               if li-FTPstatusCode = 230 then 
               run newState(40, "TYPE I").
               else 
               li-currentState = -40.
          when 40 then 
               if li-FTPstatusCode = 200 then 
               run newState(50,"NOOP").
               else 
               li-currentState = -50.
          when 50 then 
               if li-FTPstatusCode = 200 then 
               run newState(60,"PASV").
               else 
               li-currentState = -60.
          when 60 then 
               if li-FTPstatusCode = 227 and DTPconnect(lc-controlResponse) then 
               do:
                   set-size(lm-document) = 0.
                   set-size(lm-document) = li-ListBufferSize.
                   run newState(70, "NLST " + lc-rootLoadDir + "/*.xml").                
               end.              
               else 
               li-currentState = -70.
          when 70 then
               /* 150 opening data connection... */
               if li-FTPstatusCode = 150 then 
               run newState(80,"").
               else 
               /* 550 No such file or directory...*/
               if li-FTPstatusCode = 550 then 
               run newState(130,"QUIT").
               else 
               li-currentState = -80.
          when 80 then 
               /* 226 Transfer Complete... 
                  200 NOOP command successfull */
               if li-FTPstatusCode = 226 or li-FTPstatusCode = 200 then 
               do:
                   /* We don't block waiting for lists due to the 
                    * smaller amount of information that's retrived */                  
                   if li-currentFile = 0 then
                   getFileList(lm-document).

                   lc-currentFile = entry(li-currentFile,lc-FileList) no-error.

                   if error-status:error then
                   li-currentState = -90.
                   else
                   run newState(90,"PASV").                   

                   debugEvent("020","PIreadResponse","Getting file " + lc-currentFile).
               end.               
               else
               li-currentState = -90.          
          when 90 then 
               if li-FTPstatusCode = 227 and DTPconnect(lc-controlResponse) then 
               run newState(100, "RETR " + lc-currentFile).                
               else 
               li-currentState = -100.
          when 100 then 
               if li-FTPstatusCode = 150 and setBuffer(lc-controlResponse) then 
               run newState(110,"").
               else
               run newState(130,"QUIT").
               /* li-currentState = -110. */
          when 110 then
               if li-FTPstatusCode = 226 then 
               do:
                   UPLOAD:
                   repeat:
                       if li-bytesRead ge li-bytesToRead then 
                       leave UPLOAD.
                       
                       wait-for read-response of lh-DTPsocket.
                   end.

                   if xload(lm-document) then
                   run newState(115,"RNFR " + lc-currentFile).               
                   else
                   run newState(120,"NOOP").
               end.
               else
               li-currentState = -115.
          when 115 then
               /* 350 File Exists... */
               if  li-FTPstatusCode = 350 then
                   run newState(120,"RNTO " + newDirName(lc-currentFile)).
               else
                   li-currentState = -120.          
          when 120 then
               if li-FTPstatusCode = 250 or li-FTPstatusCode = 200 then              
               do:                   
                   li-currentFile = li-currentFile + 1.

                   if li-currentFile > li-filesToLoad then
                   run newState(130,"QUIT").
                   else
                   /* Force the process to loop back round */
                   run newState(80, "NOOP").
               end.              
               else
               li-currentState = -80.
          when 130 then
               if li-FTPstatusCode = 221 then               
               do:
                   assign li-currentState = 999
                          li-currentFile  = 0.

                   lh-DTPsocket:disconnect() no-error.
               end.              
               else
               li-currentState = -999.

      end case.

   end. /* ..of if li-bytesAvail > 0 then... */ 

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

/* ************************  Function Implementations ***************** */

&IF DEFINED(EXCLUDE-chomp) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION chomp Procedure 
FUNCTION chomp RETURNS CHARACTER
  ( pc-string as char, 
    pi-i      as int) :
/*------------------------------------------------------------------------------
  Purpose: Chop x characters off the end of a string, ie strip CRLF off
           the end of a line. 
    Notes:  
------------------------------------------------------------------------------*/

  return substring(pc-string,1,length(pc-string) - pi-i).

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-debugEvent) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION debugEvent Procedure 
FUNCTION debugEvent RETURNS LOGICAL
  ( input pc-ID    as char,
    input pc-title as char,
    input pc-text  as char) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  
------------------------------------------------------------------------------*/
  
/*   {&DEBUG}                                          skip  */
/*            "<ID>" pc-ID "</ID>"                     skip  */
/*            "<title>" xmlEncode(pc-title) "</title>" skip  */
/*            "<text>"  xmlEncode(pc-text)  "</text>"  skip  */
/*            "</event>"                               skip. */

  put stream debug-stream unformatted
      pc-ID " " pc-title " " pc-text skip.

  return true.

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-destroy) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION destroy Procedure 
FUNCTION destroy RETURNS LOGICAL
  ( /* void */ ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  
------------------------------------------------------------------------------*/

  message {&TSTAMP} "xloader Shutdown status=" li-currentState.

  debugEvent("000","destroy","CurrentStatus=" + string(li-currentState)).

  set-size(lm-document) = 0.

  lh-PISocket:disconnect() no-error.
  lh-DTPSocket:disconnect() no-error.

  if valid-handle(lh-PISocket) then 
      delete object lh-PISocket.
  if valid-handle(lh-DTPSocket) then 
      delete object lh-DTPSocket.
  if valid-handle(lh-xLoadLib) then 
      delete object lh-xLoadLib. 
  if valid-handle(lh-wPubLib) then 
      delete object lh-WpubLib. 

  debugEvent("999","destroy","").

  stopDebug().

  return true.

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-DTPconnect) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION DTPconnect Procedure 
FUNCTION DTPconnect RETURNS LOGICAL
  ( pc-string as character ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  
------------------------------------------------------------------------------*/
define variable lc-c                      as character                  no-undo.
define variable lc-DTP-host               as character                  no-undo.
define variable lc-DTP-port               as character                  no-undo.
define variable ll-ok                     as logical                    no-undo.
define variable li-start                  as integer                    no-undo.
define variable li-end                    as integer                    no-undo.


  assign li-bytesRead = 0
         li-start     = index(pc-string,"(":U)
         li-end       = index(pc-string,")":U)
         lc-c         = if li-start > 0 then 
                        substring(pc-string,li-start + 1,li-end - 1 - li-start)
                        else 
                        lc-c
         lc-DTP-host  = entry(1,lc-c) + "." + 
                        entry(2,lc-c) + "." + 
                        entry(3,lc-c) + "." +
                        entry(4,lc-c)
         lc-DTP-port  = string((int(entry(5,lc-c)) * 256) + int(entry(6,lc-c))) no-error.
       
         lh-DTPSocket:disconnect() no-error.

         debugEvent("000","DTPconnect","-H " + lc-DTP-host + " -S " + lc-DTP-port).

         ll-ok = lh-DTPSocket:connect("-H " + lc-DTP-host + " -S " + lc-DTP-port) no-error.

         lh-DTPSocket:set-read-response-procedure ("DTPReadResponse", THIS-PROCEDURE).

  return ll-ok.

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-getFileList) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION getFileList Procedure 
FUNCTION getFileList RETURNS LOGICAL
  ( pm-buffer as memptr ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  
------------------------------------------------------------------------------*/
define variable li-idx                    as integer                    no-undo.
define variable lc-list                   as character                  no-undo.

  assign lc-list        = get-string(pm-buffer,1)
         lc-currentFile = ""
         lc-fileList    = ""
         /* Last entry in the list is an inet address so we discard this */
         li-filesToLoad = (num-entries(lc-List,"~n") - 1).
         
  do li-idx = 1 to li-filesToLoad:
      lc-fileList = lc-fileList + "," + entry(li-idx,lc-List,"~n").
  end.

  assign lc-fileList    = substring(lc-fileList,2).                   
         li-currentFile = 1.
  set-size(pm-buffer)   = 0.

  return true.

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-initialise) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION initialise Procedure 
FUNCTION initialise RETURNS LOGICAL
  ( /* void */ ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  
------------------------------------------------------------------------------*/
  define variable ll-ok           as logical                            no-undo.

  message {&TSTAMP} "xloader Startup".

  if getConfig("xLoaderDebug") = "ON" then
  do:              
      startDebug().
      message {&TSTAMP} "Debugging information can be found in " skip lc-debugFile.
  end.
  else
      message {&TSTAMP} "Debugging is OFF".
 
  debugEvent("000","Initialise","").

  assign lc-user           = getConfig("FTPUser")
         lc-password       = getConfig("FTPPass")
         lc-FTPserverHost  = getConfig("FTPHost")        
         lc-rootLoadDir    = getConfig("FTPfldr")         
         lc-loadedDir      = getConfig("FTPloadedDir")
         lc-loadedDir      = if lc-loadedDir ne ? and lc-loadedDir ne ""
                             then lc-loadedDir
                             else
                             "loaded"
         li-listBufferSize = int(getConfig("FTPListBuffer"))
         li-listBufferSize = if li-listBufferSize > 0 then 
                             li-listBufferSize
                             else
                             8192
         li-cycleWait      = int(getConfig("FTPCycleWait"))
         li-cycleWait      = if li-cycleWait > 0 then 
                             li-cycleWait
                             else
                             30
         li-bytesRead      = 0.

  debugEvent("010","Initialise","User=" + lc-user + "," +
                                "Password=" + fill("*",length(lc-password)) + "," +
                                "FTPServerHost=" + lc-FTPServerHost + 
                                "RootLoadDir=" + lc-rootLoadDir +
                                "LoadedDir=" + lc-loadedDir +
                                "ListBuffer=" + string(li-ListBufferSize) +
                                "CycleWait=" + string(li-cycleWait)).

  /* Socket object for the protocol interpreter  */
  create socket lh-PISocket.
  /* Socket object for the Data Transfer Process */
  create socket lh-DTPSocket.
  
  LOAD-LIBS:
  do on error undo, leave:
      
      debugEvent("030","Initialise","xloadlib.p Loading...").
      
      run load/xloadlib.p persistent set lh-xloadlib.

      this-procedure:add-super-procedure(lh-xloadlib).

      debugEvent("040","Initialise","xloadlib.p Loaded Successfully").
     
      debugEvent("050","Initialise","wpublib.p Loading...").

      run load/wpublib.p persistent set lh-wpublib.

      this-procedure:add-super-procedure(lh-wpublib).

      debugEvent("060","Initialise","wpublib.p Loaded Successfully...").

  end. /* ..of LOAD-LIBS */

  PIconnect().

  debugEvent("999","Initialise","").
  
  return true.

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-newDirName) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION newDirName Procedure 
FUNCTION newDirName returns character
    (input pc-path as char):
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  
------------------------------------------------------------------------------*/
  define variable lc-path             as character                      no-undo.

    lc-path = entry(num-entries(pc-path,"/"),pc-path,"/").

    return replace(pc-path,lc-path,lc-loadedDir + "/" + lc-path).
    
end function.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-PIconnect) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION PIconnect Procedure 
FUNCTION PIconnect returns logical
  ( /* void */ ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  
------------------------------------------------------------------------------*/
  define variable ll-ok     as logical                                  no-undo.

  ll-ok = lh-PISocket:connect("-H " + lc-FTPserverHost + " -S 21") no-error.

  lh-PISocket:set-read-response-procedure ("PIReadResponse", THIS-PROCEDURE).
        
  if not ll-ok then
  do:
      debugEvent("000","connectFTPserver","Socket Connection Failed").
      return false.
  end.

  li-currentState = 10. 

  return ll-ok.

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-setBuffer) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION setBuffer Procedure 
FUNCTION setBuffer RETURNS LOGICAL
  ( pc-string as char ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  
------------------------------------------------------------------------------*/
define variable lc-c  as character no-undo.

  assign lc-c           = trim(entry(10,pc-string,chr(32))).
         li-bytesToRead = int(substring(lc-c,2,length(lc-c))) no-error.

  if error-status:error then
  return false.

  /* re-initialise from the last use */
  set-size(lm-document) = 0.
  set-size(lm-document) = li-bytesToRead.

  return true.

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-startDebug) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION startDebug Procedure 
FUNCTION startDebug returns logical
  ( /* void */ ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  
------------------------------------------------------------------------------*/

  if not ll-debug then
  do:
      assign lc-time        = string(TIME,"HH:MM:SS")
             lc-basefile    = "XLDR":U +  
                              string(TODAY,"99999999":U) + 
                              substring(lc-time,1,2) +
                              substring(lc-time,4,2) + 
                              substring(lc-time,7,2)
             lc-log         = getConfig("debugFldr")
             lc-log         = if lc-log ne ? or lc-log ne "":U then
                              lc-log
                              else
                              "/tmp":U.
             lc-debugFile   = lc-log + "/":U + lc-basefile + ".xml":U.                      
             
      output stream debug-stream to value(lc-debugFile) unbuffered.

      put stream debug-stream unformatted   
          "<?xml version=~"1.0~" standalone=~"yes~"?>"  skip
          "<logfile>"                                   skip
          "<head>"                                      skip
          "<system>OneCs</system>"                      skip
          "<filename>" lc-debugFile "</filename>"       skip
          "<desc>xLoader v{&VERSION} Debug File</desc>" skip
          "<timestamp>"                                 skip
          "<date>" string(today,"99999999")  "</date>"  skip
          "<time>" string(TIME,"HH:MM:SS":U) "</time>"  skip
          "</timestamp>"                                skip
          "</head>"                                     skip
          "<body>"                                      skip.

      ll-debug = TRUE.
  end.

  return true.   

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-stopDebug) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION stopDebug Procedure 
FUNCTION stopDebug RETURNS LOGICAL
  ( /* void */ ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  
------------------------------------------------------------------------------*/

  if ll-debug then
  do:
      {&DEBUG} "</body>"    skip
               "</logfile>" skip.

      output stream debug-stream close.
  end.

  ll-debug = FALSE.

  return true.

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-xload) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION xload Procedure 
FUNCTION xload returns logical
  ( pm-document as memptr ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  
------------------------------------------------------------------------------*/ 
  define variable ll-validDocument            as logical                no-undo.
  define variable lh-document                 as handle                 no-undo.
  define variable lh-rootNode                 as handle                 no-undo.
  define variable lc-loadProgram              as character              no-undo.
  define variable ll-success                  as logical                no-undo.

  create x-noderef  lh-rootNode.
  create x-document lh-document.

  debugEvent("000","xLoad","Parsing File " + lc-currentFile).
  
  ll-validDocument = lh-document:load("MEMPTR",lm-document,TRUE).

  if ll-validDocument then
  do:
      lh-document:get-document-element(lh-rootNode).

      if valid-handle(lh-rootNode) then
      do:
          debugEvent("010","xLoad","Root Node name=" + lh-rootNode:name).

          lc-loadProgram = getCode("xLoad",lh-rootNode:name).

          if lc-loadProgram ne "":U then
          do:
              debugEvent("020","xLoad","Running " + lc-loadProgram + " for " + lh-rootNode:name).
              run value(lc-loadProgram) (input lh-rootNode) no-error.
              if return-value = "OK" and not error-status:error
              then ll-success = TRUE.              
          end.
          else
          debugEvent("030","xLoad","Blank Load Program").
      end.
      else
      debugEvent("040","xLoad","Invalid Root Node").
  end.
  else
  debugEvent("050","xLoad","Invalid Document error=" + string(error-status:get-number(1))).

  delete object lh-document.
  delete object lh-rootNode.

  debugEvent("999","xLoad","").

  return ll-success.

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

